     package vaft.core;

     import java.io.*;
     import java.rmi.*;
     import java.util.*;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-03-15
     *********************************************************************/

     public class  PeerList implements Cloneable, Runnable, Serializable
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private transient Peer     self;
     private transient Peer     seed;
     private transient long     swap_delay;

     private transient boolean  running    = false;

     private Vector  v;

     //////////////////////////////////////////////////////////////////////
     // Constructor method
     //////////////////////////////////////////////////////////////////////

     public  PeerList ( Peer  self, Peer  seed, long  swap_delay )
     //////////////////////////////////////////////////////////////////////
     {
       this.self       = self;
       this.seed       = seed;
       this.swap_delay = swap_delay;

       v = new Vector ( );
       if ( self != null ) v.addElement ( self );
       if ( seed != null ) v.addElement ( seed );
     }

     //////////////////////////////////////////////////////////////////////
     // Access methods
     //////////////////////////////////////////////////////////////////////

     public boolean  getRunning ( )
       { return running; }
     public void     setRunning ( boolean  running )
       { this.running = running; }

     //////////////////////////////////////////////////////////////////////
     // Public instance methods
     //////////////////////////////////////////////////////////////////////

     public synchronized PeerList  swap ( PeerList  peerList )
     //////////////////////////////////////////////////////////////////////
     {
       if ( peerList == null ) return this;
       integrate ( peerList );

       // Give the peers that it does not know about.
       Enumeration  e = v.elements ( );
       PeerList  c = null;
       while ( e.hasMoreElements ( ) )
       {
         Peer  peer = ( Peer ) e.nextElement ( );
         if ( !peerList.v.contains ( peer ) )
         {
           if ( c == null ) c = new PeerList ( peer, null, swap_delay );
           else c.v.addElement ( peer );
         }
       }
       return c;
     }

     public void  run ( )
     //////////////////////////////////////////////////////////////////////
     {
       running = true;
       while ( running )
       {
         swap ( );
         try { Thread.currentThread ( ).sleep ( swap_delay ); }
         catch ( InterruptedException  ex ) { }
       }
     }

     public Object  clone ( )
     //////////////////////////////////////////////////////////////////////
     {
       return clone ( );
     }

     //////////////////////////////////////////////////////////////////////
     // Private instance methods
     //////////////////////////////////////////////////////////////////////

     private synchronized void  swap ( )
     //////////////////////////////////////////////////////////////////////
     {
       Enumeration e = v.elements ( );
       Vector  remove_Vector = new Vector ( );
       while ( e.hasMoreElements ( ) )
       {
         Peer  peer = ( Peer ) e.nextElement ( );
         if ( peer == self ) continue;
         try
         {
           PeerList  peerList = peer.swap ( this );
           integrate ( peerList );
         }
         catch ( Exception  ex )
         {
           remove_Vector.addElement ( peer );
         }
       }
       e = remove_Vector.elements ( );
       while ( e.hasMoreElements ( ) )
       {
         Peer  peer = ( Peer ) e.nextElement ( );
         if ( peer == seed ) continue;
         v.removeElement ( peer );
       }
     }

     private synchronized void  integrate ( PeerList  peerList )
     //////////////////////////////////////////////////////////////////////
     {
       // Get the peers that I do not already know about.
       Enumeration  e = peerList.v.elements ( );
       while ( e.hasMoreElements ( ) )
       {
         Peer  peer = ( Peer ) e.nextElement ( );
         if ( !v.contains ( peer ) ) v.addElement ( peer );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
